@@ -20,11 +20,20 @@
 #include <KActionCollection>
 #include <KActionMenu>
 #include <KCrash>
+#include <KHamburgerMenu>
 #include <KIconUtils>
 #include <KLocalizedString>
 #include <KShortcutsDialog>
+#include <KToolBar>
 #include <KWindowEffects>
 
+#include <kwindowsystem_version.h>
+#if HAVE_X11
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 101, 0)
+#include <KX11Extras>
+#endif
+#endif
+
 #include <KMessageBox>
 #include <KNotifyConfigWidget>
 #include <KStandardAction>
@@ -34,7 +43,6 @@
 
 #include <kio_version.h>
 #include <kwidgetsaddons_version.h>
-#include <kwindowsystem_version.h>
 
 // Konsole
 #include "BookmarkHandler.h"
@@ -84,7 +92,8 @@ MainWindow::MainWindow()
     // create view manager
     _viewManager = new ViewManager(this, actionCollection());
     connect(_viewManager, &Konsole::ViewManager::empty, this, &QWidget::close);
-    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, this, &Konsole::MainWindow::activeViewChanged);
+    // QueuedConnection so that KHamburgerMenu showed up properly on the first tab
+    connect(_viewManager, &Konsole::ViewManager::activeViewChanged, this, &Konsole::MainWindow::activeViewChanged, Qt::QueuedConnection);
     connect(_viewManager, &Konsole::ViewManager::unplugController, this, &Konsole::MainWindow::disconnectController);
     connect(_viewManager, &Konsole::ViewManager::viewPropertiesChanged, bookmarkHandler(), &Konsole::BookmarkHandler::setViews);
     connect(_viewManager, &Konsole::ViewManager::blurSettingChanged, this, &Konsole::MainWindow::setBlur);
@@ -198,7 +207,7 @@ void MainWindow::updateUseTransparency()
         return;
     }
 
-    bool useTranslucency = KWindowSystem::compositingActive();
+    bool useTranslucency = WindowSystemInfo::compositingActive();
 
     setAttribute(Qt::WA_TranslucentBackground, useTranslucency);
     setAttribute(Qt::WA_NoSystemBackground, false);
@@ -208,7 +217,18 @@ void MainWindow::updateUseTransparency()
 void MainWindow::activationRequest(const QString &xdgActivationToken)
 {
     KWindowSystem::setCurrentXdgActivationToken(xdgActivationToken);
+
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 101, 0)
+    if (KWindowSystem::isPlatformX11()) {
+#if HAVE_X11
+        KX11Extras::forceActiveWindow(winId());
+#endif
+    } else {
+        KWindowSystem::activateWindow(windowHandle());
+    }
+#else
     KWindowSystem::forceActiveWindow(winId());
+#endif
 }
 
 void MainWindow::rememberMenuAccelerators()
@@ -307,6 +327,11 @@ void MainWindow::activeViewChanged(SessionController *controller)
     connect(controller, &Konsole::SessionController::rawTitleChanged, this, &Konsole::MainWindow::updateWindowCaption);
     connect(controller, &Konsole::SessionController::iconChanged, this, &Konsole::MainWindow::updateWindowIcon);
 
+    // to prevent shortcuts conflict
+    if (auto hamburgerMenu = _hamburgerMenu->menu()) {
+        hamburgerMenu->clear();
+    }
+
     controller->setShowMenuAction(_toggleMenuBarAction);
     guiFactory()->addClient(controller);
 
@@ -400,6 +425,13 @@ void MainWindow::setupActions()
     // Settings Menu
     _toggleMenuBarAction = KStandardAction::showMenubar(menuBar(), &QMenuBar::setVisible, collection);
     collection->setDefaultShortcut(_toggleMenuBarAction, Konsole::ACCEL | Qt::Key_M);
+    connect(_toggleMenuBarAction, &QAction::triggered, [=] {
+        // Remove menubar icons set for the hamburger menu, so they don't override
+        // the text when they appear in the in-window menubar
+        collection->action(QStringLiteral("bookmark"))->setIcon(QIcon());
+        static_cast<QMenu *>(factory()->container(QStringLiteral("plugins"), this))->setIcon(QIcon());
+    });
+
     // Set up themes
     actionCollection()->addAction(QStringLiteral("window-colorscheme-menu"), new AppColorSchemeChooser(actionCollection()));
 
@@ -439,6 +471,79 @@ void MainWindow::setupActions()
             viewManager()->loadLayoutFile();
         }
     });
+
+    // Hamburger menu for when the menubar is hidden
+    _hamburgerMenu = KStandardAction::hamburgerMenu(nullptr, nullptr, collection);
+    _hamburgerMenu->setShowMenuBarAction(_toggleMenuBarAction);
+    _hamburgerMenu->setMenuBar(menuBar());
+    connect(_hamburgerMenu, &KHamburgerMenu::aboutToShowMenu, this, &MainWindow::updateHamburgerMenu);
+}
+
+void MainWindow::updateHamburgerMenu()
+{
+    KActionCollection *collection = actionCollection();
+    KActionCollection *controllerCollection = _pluggedController->actionCollection();
+
+    QMenu *menu = _hamburgerMenu->menu();
+    if (!menu) {
+        menu = new QMenu(widget());
+        _hamburgerMenu->setMenu(menu);
+    } else {
+        menu->clear();
+    }
+
+    menu->addAction(collection->action(QStringLiteral("new-window")));
+    menu->addAction(collection->action(QStringLiteral("new-tab")));
+    menu->addAction(controllerCollection->action(QStringLiteral("file_save_as")));
+
+    menu->addSeparator();
+
+    menu->addAction(controllerCollection->action(QStringLiteral("edit_copy")));
+    menu->addAction(controllerCollection->action(QStringLiteral("edit_paste")));
+    menu->addAction(controllerCollection->action(QStringLiteral("edit_find")));
+
+    menu->addSeparator();
+
+    menu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::FullScreen))));
+    menu->addAction(collection->action(QStringLiteral("split-view")));
+    menu->addAction(controllerCollection->action(QStringLiteral("clear-history-and-reset")));
+    menu->addAction(controllerCollection->action(QStringLiteral("enlarge-font")));
+    menu->addAction(controllerCollection->action(QStringLiteral("reset-font-size")));
+    menu->addAction(controllerCollection->action(QStringLiteral("shrink-font")));
+
+    menu->addSeparator();
+
+    menu->addAction(controllerCollection->action(QStringLiteral("edit-current-profile")));
+    menu->addAction(controllerCollection->action(QStringLiteral("switch-profile")));
+
+    menu->addSeparator();
+
+    auto monitorMenu = menu->addMenu(QIcon::fromTheme(QStringLiteral("visibility")), static_cast<QMenu *>(factory()->container(QStringLiteral("view"), nullptr))->title());
+    monitorMenu->addAction(controllerCollection->action(QStringLiteral("monitor-silence")));
+    monitorMenu->addAction(controllerCollection->action(QStringLiteral("monitor-activity")));
+    monitorMenu->addAction(controllerCollection->action(QStringLiteral("monitor-process-finish")));
+    menu->addMenu(monitorMenu);
+    _hamburgerMenu->hideActionsOf(monitorMenu);
+
+    auto bookmarkMenu = collection->action(QStringLiteral("bookmark"));
+    bookmarkMenu->setIcon(QIcon::fromTheme(QStringLiteral("bookmarks"))); // Icon will be removed again when the menu bar is enabled.
+    menu->addAction(bookmarkMenu);
+
+    auto pluginsMenu = static_cast<QMenu *>(factory()->container(QStringLiteral("plugins"), this));
+    pluginsMenu->setIcon(QIcon::fromTheme(QStringLiteral("plugins"))); // Icon will be removed again when the menu bar is enabled.
+    menu->addMenu(pluginsMenu);
+
+    auto configureMenu = menu->addMenu(QIcon::fromTheme(QStringLiteral("configure")), static_cast<QMenu *>(factory()->container(QStringLiteral("settings"), nullptr))->title());
+    configureMenu->addAction(toolBarMenuAction());
+    configureMenu->addSeparator();
+    configureMenu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::SwitchApplicationLanguage))));
+    configureMenu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::KeyBindings))));
+    configureMenu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::ConfigureToolbars))));
+    configureMenu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::ConfigureNotifications))));
+    configureMenu->addAction(collection->action(QLatin1String(KStandardAction::name(KStandardAction::Preferences))));
+    _hamburgerMenu->hideActionsOf(configureMenu);
+
+    _hamburgerMenu->hideActionsOf(toolBar());
 }
 
 void MainWindow::viewFullScreen(bool fullScreen)
@@ -568,6 +673,9 @@ Session *MainWindow::createSession(Profile::Ptr profile, const QString &director
     auto newView = _viewManager->createView(session);
     _viewManager->activeContainer()->addView(newView);
 
+    _viewManager->activeViewController()->actionCollection()->addActions(
+        {_hamburgerMenu});
+
     return session;
 }
 
@@ -655,8 +763,14 @@ bool MainWindow::queryClose()
     // NOTE: Some, if not all, of the below KWindowSystem calls are only
     //       implemented under x11 (KDE4.8 kdelibs/kdeui/windowmanagement).
 
+#if HAVE_X11
     // make sure the window is shown on current desktop and is not minimized
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 101, 0)
+    KX11Extras::setOnDesktop(winId(), KX11Extras::currentDesktop());
+#else
     KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());
+#endif
+#endif
     int result;
 
     if (!processesRunning.isEmpty()) {
@@ -962,18 +1076,35 @@ void MainWindow::setRemoveWindowTitleBarAndFrame(bool frameless)
 
         // The window is visible and the setting changed
     } else if (windowFlags().testFlag(Qt::FramelessWindowHint) != frameless) {
-        const auto oldGeometry = saveGeometry();
-        // This happens for every Konsole window. It depends on
-        // the fact that every window is processed in single thread
-        const auto oldActiveWindow = KWindowSystem::activeWindow();
+        if (KWindowSystem::isPlatformX11()) {
+#if HAVE_X11
+            const auto oldGeometry = saveGeometry();
+            // This happens for every Konsole window. It depends on
+            // the fact that every window is processed in single thread
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 101, 0)
+            const auto oldActiveWindow = KX11Extras::activeWindow();
+#else
+            const auto oldActiveWindow = KWindowSystem::activeWindow();
+#endif
 
-        setWindowFlags(newFlags);
+            setWindowFlags(newFlags);
 
-        // The setWindowFlags() has hidden the window. Show it again
-        // with previous geometry
-        restoreGeometry(oldGeometry);
-        setVisible(true);
-        KWindowSystem::activateWindow(oldActiveWindow);
+            // The setWindowFlags() has hidden the window. Show it again
+            // with previous geometry
+            restoreGeometry(oldGeometry);
+            setVisible(true);
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 101, 0)
+            KX11Extras::activateWindow(oldActiveWindow);
+#else
+            KWindowSystem::activateWindow(oldActiveWindow);
+#endif
+#endif
+        } else {
+            // Restoring geometry ourselves doesn't work on Wayland
+            setWindowFlags(newFlags);
+            // The setWindowFlags() has hidden the window. Show it again
+            setVisible(true);
+        }
     }
 }
 
