@@ -21,12 +21,12 @@
 
 #include "pluginsystem/IKonsolePlugin.h"
 
-#include "konsole_export.h"
+#include "konsoleapp_export.h"
 
 #include <vector>
 
-class QAction;
 class KActionMenu;
+class KHamburgerMenu;
 class KToggleAction;
 
 namespace Konsole
@@ -49,7 +49,7 @@ class BookmarkHandler;
  *
  * Do not construct new main windows directly, use Application's newMainWindow() method.
  */
-class KONSOLE_EXPORT MainWindow : public KXmlGuiWindow
+class KONSOLEAPP_EXPORT MainWindow : public KXmlGuiWindow
 {
     Q_OBJECT
 
@@ -202,12 +202,14 @@ private:
     // sets the active shortcuts of actions in 'dest' to the shortcuts of actions
     // with the same name in 'source' (see QAction::ActiveShortcut)
     static void syncActiveShortcuts(KActionCollection *dest, const KActionCollection *source);
+    void updateHamburgerMenu();
 
 private:
     ViewManager *_viewManager;
     BookmarkHandler *_bookmarkHandler;
     KToggleAction *_toggleMenuBarAction;
     KActionMenu *_newTabMenuAction;
+    KHamburgerMenu *_hamburgerMenu;
 
     QPointer<SessionController> _pluggedController;
     std::vector<IKonsolePlugin *> _plugins;
