@@ -91,6 +91,7 @@ Screen::Screen(int lines, int columns)
     , _lastPos(-1)
     , _lastDrawnChar(0)
     , _escapeSequenceUrlExtractor(nullptr)
+    , _ignoreWcWidth(false)
 {
     std::fill(_lineProperties.begin(), _lineProperties.end(), LineProperty());
 
@@ -551,6 +552,11 @@ void Screen::setReflowLines(bool enable)
     _enableReflowLines = enable;
 }
 
+void Screen::setIgnoreWcWidth(bool ignore)
+{
+    _ignoreWcWidth = ignore;
+}
+
 /* Note that if you use these debugging functions, it will
    fail to compile on gcc 8.3.1 as of Feb 2021 due to for_each_n().
    See BKO: 432639
@@ -632,14 +638,14 @@ void Screen::resizeImage(int new_lines, int new_columns)
             auto sessionController = _currentTerminalDisplay->sessionController();
             auto terminal = sessionController->session()->foregroundProcessName();
             if (terminal == QLatin1String("zsh")) {
-                while (cursorLine + cursorLineCorrection > 0 && (_lineProperties.at(cursorLine + cursorLineCorrection).flags.f.prompt_start) == 0) {
+                while (cursorLine + cursorLineCorrection > 0 && (linePropertiesAt(cursorLine + cursorLineCorrection).flags.f.prompt_start) == 0) {
                     --cursorLineCorrection;
                 }
-                if (cursorLine + cursorLineCorrection > 0 && (_lineProperties.at(cursorLine + cursorLineCorrection).flags.f.prompt_start) != 0) {
+                if (cursorLine + cursorLineCorrection > 0 && (linePropertiesAt(cursorLine + cursorLineCorrection).flags.f.prompt_start) != 0) {
                     _lineProperties[cursorLine + cursorLineCorrection - 1].flags.f.wrapped = 0;
                 } else {
                     cursorLineCorrection = 0;
-                    while (cursorLine + cursorLineCorrection > 0 && (_lineProperties.at(cursorLine + cursorLineCorrection - 1).flags.f.wrapped) != 0) {
+                    while (cursorLine + cursorLineCorrection > 0 && (linePropertiesAt(cursorLine + cursorLineCorrection - 1).flags.f.wrapped) != 0) {
                         --cursorLineCorrection;
                     }
                 }
@@ -1074,10 +1080,10 @@ void Screen::displayCharacter(uint c)
     if (w < 0) {
         // Non-printable character
         return;
-    } else if (category == QChar::Mark_SpacingCombining || w == 0 || Character::emoji(c) || c == 0x20E3) {
+    } else if (category == QChar::Mark_SpacingCombining || w == 0 || Character::emoji(c) || c == 0x20E3 || (_ignoreWcWidth && c == 0x00AD)) {
         bool emoji = Character::emoji(c);
         if (category != QChar::Mark_SpacingCombining && category != QChar::Mark_NonSpacing && category != QChar::Letter_Other && category != QChar::Other_Format
-            && !emoji && c != 0x20E3) {
+            && !emoji && c != 0x20E3 && c != 0x00AD) {
             return;
         }
         // Find previous "real character" to try to combine with
@@ -1962,7 +1968,7 @@ QString Screen::text(int startIndex, int endIndex, const DecodingOptions options
     QString result;
     QTextStream stream(&result, QIODevice::ReadWrite);
 
-    HTMLDecoder htmlDecoder;
+    HTMLDecoder htmlDecoder(ColorScheme::defaultTable);
     PlainTextDecoder plainTextDecoder;
 
     TerminalCharacterDecoder *decoder;
@@ -2065,7 +2071,8 @@ int Screen::copyLineToStream(int line,
     // determine if the line is in the history buffer or the screen image
     if (line < _history->getLines()) {
         // ensure that start position is before end of line
-        start = qBound(0, start, lineLength - 1);
+        // lineLength can be 0 as well
+        start = lineLength <= 0 ? 0 : qBound(0, start, lineLength - 1);
 
         // retrieve line from history buffer
         if (count == -1) {
@@ -2138,7 +2145,8 @@ int Screen::copyLineToStream(int line,
         }
 
         // count cannot be any greater than length
-        count = qBound(0, count, length - start);
+        // and if start is after length we have nothing to copy
+        count = start >= length ? 0 : qBound(0, count, length - start);
 
         Q_ASSERT((size_t)screenLine < _lineProperties.size());
         currentLineProperties = _lineProperties[screenLine];
@@ -2209,14 +2217,13 @@ void Screen::fastAddHistLine()
 {
     const bool removeLine = _history->getLines() == _history->getMaxLines();
     _history->addCellsVector(_screenLines.at(0));
-    _history->addLine(_lineProperties.at(0));
+    _history->addLine(linePropertiesAt(0));
 
     // If _history size > max history size it will drop a line from _history.
     // We need to verify if we need to remove a URL.
     if (removeLine && _escapeSequenceUrlExtractor) {
         _escapeSequenceUrlExtractor->historyLinesRemoved(1);
     }
-
     // Rotate left + clear the last line
     std::rotate(_screenLines.begin(), _screenLines.begin() + 1, _screenLines.end());
     auto last = _screenLines.back();
@@ -2335,6 +2342,14 @@ void Screen::setLineProperty(quint16 property, bool enable)
     }
 }
 
+LineProperty Screen::linePropertiesAt(unsigned int line)
+{
+    if (line < _lineProperties.size()) {
+        return _lineProperties.at(line);
+    }
+    return LineProperty();
+}
+
 void Screen::setReplMode(int mode)
 {
     if (_replMode != mode) {
