@@ -18,29 +18,32 @@
 #include "profile/ProfileModel.h"
 
 #include "sshmanagerfiltermodel.h"
+#include "sshmanagerplugin.h"
 #include "ui_sshwidget.h"
 
 #include <KLocalizedString>
+#include <KMessageBox>
+#include <kwidgetsaddons_version.h>
 
 #include <QAction>
-#include <QDebug>
 #include <QFileDialog>
 #include <QIntValidator>
 #include <QItemSelectionModel>
 #include <QMenu>
-#include <QMessageBox>
 #include <QPoint>
 #include <QRegularExpression>
 #include <QRegularExpressionValidator>
-#include <QSortFilterProxyModel>
-#include <QStandardPaths>
 
-#include <KMessageBox>
+#include <QSettings>
+#include <QSortFilterProxyModel>
+
+#include <QStandardPaths>
 
 struct SSHManagerTreeWidget::Private {
     SSHManagerModel *model = nullptr;
     SSHManagerFilterModel *filterModel = nullptr;
     Konsole::SessionController *controller = nullptr;
+    bool isSetup = false;
 };
 
 SSHManagerTreeWidget::SSHManagerTreeWidget(QWidget *parent)
@@ -85,8 +88,9 @@ SSHManagerTreeWidget::SSHManagerTreeWidget(QWidget *parent)
         d->filterModel->invalidate();
     });
 
-    ui->profile->setModel(Konsole::ProfileModel::instance());
-    ui->profile->setModelColumn(Konsole::ProfileModel::PROFILE);
+    connect(Konsole::ProfileModel::instance(), &Konsole::ProfileModel::rowsRemoved, this, &SSHManagerTreeWidget::updateProfileList);
+    connect(Konsole::ProfileModel::instance(), &Konsole::ProfileModel::rowsInserted, this, &SSHManagerTreeWidget::updateProfileList);
+    updateProfileList();
 
     ui->treeView->setContextMenuPolicy(Qt::CustomContextMenu);
 
@@ -119,7 +123,10 @@ SSHManagerTreeWidget::SSHManagerTreeWidget(QWidget *parent)
         menu->popup(ui->treeView->viewport()->mapToGlobal(pos));
     });
 
-    connect(ui->treeView, &QTreeView::doubleClicked, this, &SSHManagerTreeWidget::connectRequested);
+    connect(ui->treeView, &QTreeView::doubleClicked, this, [this](const QModelIndex &idx) {
+        SSHManagerPlugin::requestConnection(d->filterModel, d->model, d->controller, idx);
+    });
+
     connect(ui->treeView, &SshTreeView::mouseButtonClicked, this, &SSHManagerTreeWidget::handleTreeClick);
 
     ui->treeView->setModel(d->filterModel);
@@ -128,10 +135,39 @@ SSHManagerTreeWidget::SSHManagerTreeWidget(QWidget *parent)
     ui->btnEdit->setEnabled(false);
 
     clearSshInfo();
+
+    QSettings settings;
+    settings.beginGroup(QStringLiteral("plugins"));
+    settings.beginGroup(QStringLiteral("sshplugin"));
+
+    const QKeySequence def(Qt::CTRL | Qt::ALT | Qt::Key_H);
+    const QString defText = def.toString();
+    const QString entry = settings.value(QStringLiteral("ssh_shortcut"), defText).toString();
+    const QKeySequence shortcutEntry(entry);
+
+    connect(ui->keySequenceEdit, &QKeySequenceEdit::keySequenceChanged, this, [this] {
+        auto shortcut = ui->keySequenceEdit->keySequence();
+        Q_EMIT quickAccessShortcutChanged(shortcut);
+    });
+    ui->keySequenceEdit->setKeySequence(shortcutEntry);
 }
 
 SSHManagerTreeWidget::~SSHManagerTreeWidget() = default;
 
+void SSHManagerTreeWidget::updateProfileList()
+{
+    ui->profile->clear();
+    ui->profile->addItem(i18n("Don't Change"));
+    auto model = Konsole::ProfileModel::instance();
+    for (int i = 0, end = model->rowCount(QModelIndex()); i < end; i++) {
+        const int column = Konsole::ProfileModel::Column::PROFILE;
+        const int role = Qt::DisplayRole;
+        const QModelIndex currIdx = model->index(i, column);
+        const auto profileName = model->data(currIdx, role).toString();
+        ui->profile->addItem(profileName);
+    }
+}
+
 void SSHManagerTreeWidget::addSshInfo()
 {
     SSHConfigurationData data;
@@ -198,14 +234,22 @@ void SSHManagerTreeWidget::triggerDelete()
     const QString dontAskAgainKey =
         ui->treeView->model()->rowCount(selection.at(0)) ? QStringLiteral("remove_ssh_folder") : QStringLiteral("remove_ssh_config");
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    int result = KMessageBox::warningTwoActions(this,
+#else
     int result = KMessageBox::warningYesNo(this,
-                                           dialogMessage,
-                                           i18nc("@title:window", "Delete SSH Configurations"),
-                                           KStandardGuiItem::del(),
-                                           KStandardGuiItem::cancel(),
-                                           dontAskAgainKey);
+#endif
+                                                dialogMessage,
+                                                i18nc("@title:window", "Delete SSH Configurations"),
+                                                KStandardGuiItem::del(),
+                                                KStandardGuiItem::cancel(),
+                                                dontAskAgainKey);
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    if (result == KMessageBox::ButtonCode::SecondaryAction) {
+#else
     if (result == KMessageBox::ButtonCode::No) {
+#endif
         return;
     }
 
@@ -231,7 +275,11 @@ void SSHManagerTreeWidget::editSshInfo()
     ui->name->setText(data.name);
     ui->port->setText(data.port);
     ui->sshkey->setText(data.sshKey);
-    ui->profile->setCurrentText(data.profileName);
+    if (data.profileName.isEmpty()) {
+        ui->profile->setCurrentIndex(0);
+    } else {
+        ui->profile->setCurrentText(data.profileName);
+    }
     ui->username->setText(data.username);
     ui->useSshConfig->setCheckState(data.useSshConfig ? Qt::Checked : Qt::Unchecked);
 
@@ -324,6 +372,8 @@ void SSHManagerTreeWidget::setModel(SSHManagerModel *model)
     d->model = model;
     d->filterModel->setSourceModel(model);
     ui->folder->addItems(d->model->folders());
+    ui->btnManageProfile->setChecked(d->model->getManageProfile());
+    connect(ui->btnManageProfile, &QPushButton::clicked, d->model, &SSHManagerModel::setManageProfile);
 }
 
 void SSHManagerTreeWidget::setCurrentController(Konsole::SessionController *controller)
@@ -424,79 +474,14 @@ void SSHManagerTreeWidget::handleTreeClick(Qt::MouseButton btn, const QModelInde
         }
 
         Q_EMIT requestNewTab();
-        connectRequested(idx);
+        SSHManagerPlugin::requestConnection(d->filterModel, d->model, d->controller, sourceIdx);
     }
 }
 
-void SSHManagerTreeWidget::connectRequested(const QModelIndex &idx)
+void SSHManagerTreeWidget::showEvent(QShowEvent *)
 {
-    if (!d->controller) {
-        return;
-    }
-
-    auto sourceIdx = d->filterModel->mapToSource(idx);
-    if (sourceIdx.parent() == d->model->invisibleRootItem()->index()) {
-        return;
-    }
-
-    Konsole::ProcessInfo *info = d->controller->session()->getProcessInfo();
-    bool ok = false;
-    QString processName = info->name(&ok);
-    if (!ok) {
-        KMessageBox::messageBox(this,
-                                KMessageBox::DialogType::Sorry,
-                                i18n("Could not get the process name, assume that we can't request a connection"),
-                                i18n("Error issuing SSH Command"),
-                                KStandardGuiItem::yes(),
-                                KStandardGuiItem::no(),
-                                KStandardGuiItem::cancel(),
-                                QStringLiteral("error_process_name"));
-        return;
-    }
-
-    if (!QVector<QString>({QStringLiteral("fish"),
-                           QStringLiteral("bash"),
-                           QStringLiteral("dash"),
-                           QStringLiteral("sh"),
-                           QStringLiteral("csh"),
-                           QStringLiteral("ksh"),
-                           QStringLiteral("zsh")})
-             .contains(processName)) {
-        KMessageBox::messageBox(this,
-                                KMessageBox::DialogType::Sorry,
-                                i18n("Can't issue SSH command outside the shell application (eg, bash, zsh, sh)"),
-                                i18n("Error issuing SSH Command"),
-                                KStandardGuiItem::yes(),
-                                KStandardGuiItem::no(),
-                                KStandardGuiItem::cancel(),
-                                QStringLiteral("error_process_not_shell"));
-        return;
-    }
-
-    auto item = d->model->itemFromIndex(sourceIdx);
-    auto data = item->data(SSHManagerModel::SSHRole).value<SSHConfigurationData>();
-
-    QString sshCommand = QStringLiteral("ssh ");
-    if (!data.useSshConfig) {
-        if (data.sshKey.length()) {
-            sshCommand += QStringLiteral("-i %1 ").arg(data.sshKey);
-        }
-
-        if (data.port.length()) {
-            sshCommand += QStringLiteral("-p %1 ").arg(data.port);
-        }
-
-        if (!data.username.isEmpty()) {
-            sshCommand += data.username + QLatin1Char('@');
-        }
-    }
-
-    if (!data.host.isEmpty()) {
-        sshCommand += data.host;
-    }
-
-    d->controller->session()->sendTextToTerminal(sshCommand, QLatin1Char('\r'));
-    if (d->controller->session()->views().count()) {
-        d->controller->session()->views().at(0)->setFocus();
+    if (!d->isSetup) {
+        ui->treeView->expandAll();
+        d->isSetup = true;
     }
 }
