@@ -13,7 +13,10 @@
 // Standard
 #include <csignal>
 #include <cstdlib>
+
+#ifndef Q_OS_WIN
 #include <unistd.h>
+#endif
 
 // Qt
 #include <QApplication>
@@ -28,8 +31,12 @@
 #include <KLocalizedString>
 #include <KNotification>
 #include <KProcess>
+
+#ifndef Q_OS_WIN
 #include <KPtyDevice>
+#endif
 #include <KShell>
+
 #include <kcoreaddons_version.h>
 
 // Konsole
@@ -52,6 +59,7 @@
 #include "terminalDisplay/TerminalFonts.h"
 #include "terminalDisplay/TerminalScrollBar.h"
 
+#ifndef Q_OS_WIN
 // Linux
 #if HAVE_GETPWUID
 #include <pwd.h>
@@ -61,6 +69,7 @@
 #if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 97, 0)
 #include <KSandbox>
 #endif
+#endif // Q_OS_WIN
 
 using namespace Konsole;
 
@@ -144,7 +153,11 @@ void Session::openTeletype(int fd, bool runShell)
     connect(_emulation, &Konsole::Emulation::useUtf8Request, _shellProcess, &Konsole::Pty::setUtf8Mode);
 
     // get notified when the pty process is finished
+#ifndef Q_OS_WIN
     connect(_shellProcess, QOverload<int, QProcess::ExitStatus>::of(&Konsole::Pty::finished), this, &Konsole::Session::done);
+#else
+    connect(_shellProcess, &Konsole::Pty::finished, this, &Konsole::Session::done);
+#endif
 
     // emulator size
     connect(_emulation, &Konsole::Emulation::imageSizeChanged, this, &Konsole::Session::updateWindowSize);
@@ -196,7 +209,11 @@ void Session::setDarkBackground(bool darkBackground)
 
 bool Session::isRunning() const
 {
+#ifdef Q_OS_WIN
+    return (_shellProcess != nullptr) && _shellProcess->isRunning();
+#else
     return (_shellProcess != nullptr) && (_shellProcess->state() == QProcess::Running);
+#endif
 }
 
 bool Session::hasFocus() const
@@ -361,6 +378,22 @@ QString Session::checkProgram(const QString &program)
         return QString();
     }
 
+#ifndef Q_OS_WIN
+#if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 97, 0)
+    if (KSandbox::isFlatpak()) {
+        QProcess proc;
+        // run "test -x exec" on the host to see if the shell is executable
+        proc.setProgram(QStringLiteral("test"));
+        proc.setArguments(QStringList{QStringLiteral("-x"), exec});
+        KSandbox::startHostProcess(proc, QProcess::ReadOnly);
+        if (proc.waitForStarted() && proc.waitForFinished(-1)) {
+            return proc.exitCode() == 0 ? exec : QString();
+        }
+        return {};
+    }
+#endif
+#endif // Q_OS_WIN
+
     QFileInfo info(exec);
     if (info.isAbsolute() && info.exists() && info.isExecutable()) {
         return exec;
@@ -401,6 +434,29 @@ QString Session::shellSessionId() const
     return friendlyUuid;
 }
 
+static QStringList postProcessArgs(const QStringList &args)
+{
+#ifndef Q_OS_WIN
+    if (!KSandbox::isFlatpak()) {
+        return args;
+    }
+    QStringList arguments;
+    // last arg is the program
+    arguments = args;
+    QString program = arguments.back();
+    arguments.pop_back();
+    // Needs to be explicitly specified, KSandbox::makeHostContext
+    // ignores any variables set in the system environment so this
+    // never gets set.
+    arguments.push_back(QStringLiteral("--env=TERM=xterm-256color"));
+    arguments.push_back(program);
+    return arguments;
+#else
+    qCritical() << "Should never get called on windows";
+    return {};
+#endif
+}
+
 void Session::run()
 {
     // FIXME: run() is called twice in some instances
@@ -481,11 +537,16 @@ void Session::run()
 
     _shellProcess->setFlowControlEnabled(_flowControlEnabled);
     _shellProcess->setEraseChar(_emulation->eraseChar());
+#ifndef Q_OS_WIN
     _shellProcess->setUseUtmp(_addToUtmp);
+#endif
+
+#ifndef Q_OS_WIN
 #if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 97, 0)
     if (KSandbox::isFlatpak()) {
         _shellProcess->pty()->setCTtyEnabled(false); // not possibly inside sandbox
     }
+#endif
 #endif
 
     // this is not strictly accurate use of the COLORFGBG variable.  This does not
@@ -505,18 +566,27 @@ void Session::run()
     const QString dbusObject = QStringLiteral("/Sessions/%1").arg(QString::number(_sessionId));
     addEnvironmentEntry(QStringLiteral("KONSOLE_DBUS_SESSION=%1").arg(dbusObject));
 
+#ifndef Q_OS_WIN
 #if KCOREADDONS_VERSION >= QT_VERSION_CHECK(5, 97, 0)
     const auto originalEnvironment = _shellProcess->environment();
     _shellProcess->setProgram(exec);
     _shellProcess->setArguments(arguments);
     _shellProcess->setEnvironment(originalEnvironment + _environment);
     const auto context = KSandbox::makeHostContext(*_shellProcess);
+    arguments = postProcessArgs(context.arguments);
     _shellProcess->setEnvironment(originalEnvironment);
-    const auto result = _shellProcess->start(context.program, context.arguments, _environment);
+    const auto result = _shellProcess->start(context.program, arguments, _environment);
 #else
     int result = _shellProcess->start(exec, arguments, _environment);
 #endif
 
+#else // Q_OS_WIN
+    const auto size = _emulation->imageSize();
+    const int lines = size.height();
+    const int cols = size.width();
+    int result = _shellProcess->start(exec, arguments, _initialWorkingDir.isEmpty() ? QDir::currentPath() : _initialWorkingDir, _environment, cols, lines);
+#endif
+
     if (result < 0) {
         terminalWarning(i18n("Could not start program '%1' with arguments '%2'.", exec, arguments.join(QLatin1String(" "))));
         terminalWarning(_shellProcess->errorString());
@@ -800,6 +870,7 @@ void Session::refresh()
 
 void Session::sendSignal(int signal)
 {
+#ifndef Q_OS_WIN
     const ProcessInfo *process = getProcessInfo();
     bool ok = false;
     int pid;
@@ -810,6 +881,9 @@ void Session::sendSignal(int signal)
     } else {
         qWarning() << "foreground process id not set, unable to send signal " << signal;
     }
+#else
+    // FIXME: Can we do this on windows?
+#endif
 }
 
 void Session::reportColor(SessionAttributes r, const QColor &c, uint terminator)
@@ -839,6 +913,7 @@ void Session::reportBackgroundColor(const QColor &c, uint terminator)
 
 bool Session::kill(int signal)
 {
+#ifndef Q_OS_WIN
     if (processId() <= 0) {
         return false;
     }
@@ -850,6 +925,9 @@ bool Session::kill(int signal)
     } else {
         return false;
     }
+#else
+    return false;
+#endif
 }
 
 void Session::close()
@@ -868,6 +946,10 @@ void Session::close()
 
 bool Session::closeInNormalWay()
 {
+#ifdef Q_OS_WIN
+    _shellProcess->closePty();
+    return true;
+#else
     _autoClose = true;
     _closePerUserRequest = true;
 
@@ -892,6 +974,7 @@ bool Session::closeInNormalWay()
     qWarning() << "Process " << processId() << " did not die with SIGHUP";
     _shellProcess->closePty();
     return (_shellProcess->waitForFinished(1000));
+#endif
 }
 
 bool Session::closeInForceWay()
@@ -899,12 +982,16 @@ bool Session::closeInForceWay()
     _autoClose = true;
     _closePerUserRequest = true;
 
+#ifdef Q_OS_WIN
+    return _shellProcess->kill();
+#else
     if (kill(SIGKILL)) {
         return true;
     } else {
         qWarning() << "Process " << processId() << " did not die with SIGKILL";
         return false;
     }
+#endif
 }
 
 void Session::sendTextToTerminal(const QString &text, const QChar &eol) const
@@ -963,8 +1050,12 @@ void Session::sendMouseEvent(int buttons, int column, int line, int eventType)
 
 void Session::done(int exitCode, QProcess::ExitStatus exitStatus)
 {
+#ifndef Q_OS_WIN
     // This slot should be triggered only one time
     disconnect(_shellProcess, QOverload<int, QProcess::ExitStatus>::of(&Konsole::Pty::finished), this, &Konsole::Session::done);
+#else
+    disconnect(_shellProcess, &Konsole::Pty::finished, this, &Konsole::Session::done);
+#endif
 
     if (!_autoClose) {
         _userTitle = i18nc("@info:shell This session is done", "Finished");
