@@ -213,7 +213,6 @@ void TerminalPainter::drawContents(Character *image,
     const QFont::Weight normalWeight = static_cast<QFont::Weight>(m_parentDisplay->font().weight()); // Qt6: cast can go away
     auto it = std::upper_bound(std::begin(FontWeights), std::end(FontWeights), normalWeight);
     const QFont::Weight boldWeight = it != std::end(FontWeights) ? *it : QFont::Black;
-    paint.setRenderHint(QPainter::Antialiasing, m_parentDisplay->terminalFont()->antialiasText());
     paint.setLayoutDirection(Qt::LeftToRight);
     const QColor *colorTable = m_parentDisplay->terminalColor()->colorTable();
 
@@ -1149,7 +1148,7 @@ void TerminalPainter::drawTextCharacters(QPainter &painter,
         }
         painter.drawText(rect.x(), y, text);
         if (0 && text.toUcs4().length() >= 1) {
-            fprintf(stderr, " %i  ", text.toUcs4().length());
+            fprintf(stderr, " %lli  ", (qint64)text.toUcs4().length());
             for (int i = 0; i < text.toUcs4().length(); i++) {
                 fprintf(stderr, " %04x  ", text.toUcs4()[i]);
             }
